#version 430 core

layout (location=0) out vec4 fragColor;

in VertexData {
	vec3 positionWS;
	vec3 normalWS;
	vec2 UV;
} fs_in;

uniform vec3 eyePos;
uniform vec3 dirLight_direction = vec3(0,0,-1); // normalized!
uniform float ambientLight = 0.1;

uniform float mat_diffuse  = 0.7;
uniform float mat_specular = 0.3;
uniform float mat_shiny = 8.0;

uniform float u_scale = 1.0;
uniform int uv_mode = 0;

uniform sampler2D texVParam;

void main() {
	vec3 color = vec3(1,1,0);

	if (uv_mode == 1) {
		#if 0
		// scalar field
		float uNormed = fs_in.UV.x * u_scale;
		color = mix(vec3(0,0,1), vec3(0,1,0), uNormed);
		// with iso-lines
		float f = fract(uNormed * 100.0);
		if (f < 0.2) color = mix(vec3(0,0,0), color, abs(0.1-f) * 10.0);
		#endif

		#if 1
		{
		// sample V-texture for V
		float v = texelFetch(texVParam, ivec2(floor(gl_FragCoord.xy)), 0).r;
		if (v <= 0) { fragColor = vec4(1,0,0,1); return; }
		float f = fract(v * 10.0);
		if (f < 0.2) color = mix(vec3(0,0,0), color, abs(0.1-f) * 10.0);
		}
		#endif

	}

	// blinn-phong
	vec3 vL = -dirLight_direction;
	vec3 vV = normalize(eyePos - fs_in.positionWS);
	vec3 vH = normalize(vL + vV);
	vec3 vN = normalize(fs_in.normalWS);
	float diff = max(dot(vL, vN), 0);
	float spec = pow(max(dot(vH, vN), 0.0), mat_shiny);
	float illum = ambientLight + diff * mat_diffuse + spec * mat_specular;
	fragColor = vec4(color * illum, 1);


	//fragColor = vec4(1,0,0,1);
	//fragColor = vec4(normalize(fs_in.normalWS) * 0.5 + 0.5, 1);
}
